package cz.muni.fi.pb162.test3;

/**
 * Toto rozhrani reprezentuje zeleznicni stanici. Stanice ma urcity pocet koleji,
 * na kazde koleji smi byt v urcitem okamziku maximalne jeden vlak.
 *
 * @author Petr Adamek
 */
public interface RailwayStation {
    
    /**
     * Vraci pocet koleji ve stanici.
     *
     * @return Pocet koleji ve stanici.
     */
    int getTracksCount();
    
    /**
     * Vraci pocet neobsazenych koleji ve stanici.
     *
     * @return pocet neobsazenych koleji ve stanici
     */
    int getFreeTracksCount();
    
    /**
     * Tato operace reprezentuje vjezd vlaku do stanice. Vlak vjede na volnou kolej s 
     * nejnizsim cislem.
     *
     * @param train vlak, ktery vjizdi do stanice
     * @throws java.lang.NullPointerException pokud je train null
     * @throws cz.muni.fi.pb162.test2.NoFreeTrackException pokud ve stanici neni 
     * zadna volna kolej
     * @throws java.lang.IllegalArgumentException pokud jiz dany vlak 
     * ve stanici je
     */
    void enter(Train train) throws NullPointerException, 
            NoFreeTrackException, IllegalArgumentException;
    
    /**
     * Tato operace reprezentuje odjezd vlaku ze stanice.
     *
     * @param train vlak, ktery ma odjet
     * @throws java.lang.NullPointerException pokud je train null
     * @throws java.lang.IllegalArgumentException pokud dany vlak ve stanici neni
     */
    void leave(Train train) throws NullPointerException, 
            IllegalArgumentException;

    /**
     * Vraci cislo koleje, na ktere se nachazi dany vlak. Pokud se vlak ve
     * stanici nenachazi, vraci nulu. Pozor, koleje jsou cislovany od jedne!
     *
     * @return cislo koleje, na ktere se nachazi dany vlak, nebo nula, pokud
     * dany vlak ve stanici neni.
     * @param train vlak
     * @throws java.lang.NullPointerException pokud je train null
     */    
    int getTrackNumber(Train train) throws NullPointerException;
    
    /**
     * Vrati vlak, ktery se nachazi na dane koleji. 
     * Pozor, koleje jsou cislovany od jedne!
     *
     * @param trackNumber cislo koleje
     * @throws java.lang.IndexOutOfBoundsException pokud je cislo koleje mimo 
     * platny rozsah.
     * @return vlak, ktery se nachazi na dane koleji, nebo null pokud tam zadny
     * vlak neni.
     */
    Train getTrain(int trackNumber) throws IndexOutOfBoundsException;
    
    /**
     * Vrati retezec popisujici nadrazi v predepsanem formatu:
     * 
     * Tracks: &lt;pocetKoleji&gt;, Free Tracks: &lt;pocetVolnychKoleji&gt; 
     * [1: &lt;vlakNaPrvniKoleji&gt;, 2: (none), 3: &lt;vlakNaTretiKoleji&gt;]
     * 
     * @return Popis nadrazi ve stanovenem formatu.
     */
    String toString();
    
}
